@@ -224,6 +224,9 @@ else:
     config_file = config_file.replace('\'{{{ EMSCRIPTEN_ROOT }}}\'', repr(__rootpath__))
     llvm_root = os.path.dirname(find_executable('llvm-dis') or '/usr/bin/llvm-dis')
     config_file = config_file.replace('\'{{{ LLVM_ROOT }}}\'', repr(llvm_root))
+    binaryen_root = os.path.dirname(find_executable('asm2wasm') or '/usr/bin/asm2wasm')
+    config_file = config_file.replace('\'{{{ BINARYEN_ROOT }}}\'', repr(binaryen_root))
+
     node = find_executable('nodejs') or find_executable('node') or 'node'
     config_file = config_file.replace('\'{{{ NODE }}}\'', repr(node))
     if WINDOWS:
@@ -977,9 +980,12 @@ class Settings2(type):
       for i in range(len(args)):
         if args[i].startswith('-O'):
           v = args[i][2]
-          if v in ['s', 'z']: v = '2'
+          shrink = 0
+          if v in ['s', 'z']:
+            v = '2'
+            shrink = 1 if v == 's' else 2
           level = eval(v)
-          self.apply_opt_level(level)
+          self.apply_opt_level(level, shrink)
       for i in range(len(args)):
         if args[i] == '-s':
           declare = re.sub(r'([\w\d]+)\s*=\s*(.+)', r'self.attrs["\1"]=\2;', args[i+1])
@@ -1001,12 +1007,14 @@ class Settings2(type):
       self.attrs = values
 
     @classmethod
-    def apply_opt_level(self, opt_level, noisy=False):
+    def apply_opt_level(self, opt_level, shrink_level=0, noisy=False):
       if opt_level >= 1:
         self.attrs['ASM_JS'] = 1
         self.attrs['ASSERTIONS'] = 0
         self.attrs['DISABLE_EXCEPTION_CATCHING'] = 1
         self.attrs['ALIASING_FUNCTION_POINTERS'] = 1
+      if shrink_level >= 2:
+        self.attrs['EVAL_CTORS'] = 1
 
     def __getattr__(self, attr):
       if attr in self.attrs:
@@ -1654,11 +1662,71 @@ class Building:
       ret = output_filename
     return ret
 
+  @staticmethod
+  def eval_ctors(js_file, mem_init_file):
+    subprocess.check_call([PYTHON, path_from_root('tools', 'ctor_evaller.py'), js_file, mem_init_file, str(Settings.TOTAL_MEMORY), str(Settings.TOTAL_STACK), str(Settings.GLOBAL_BASE)])
+
   @staticmethod
   def eliminate_duplicate_funcs(filename):
     import duplicate_function_eliminator
     duplicate_function_eliminator.eliminate_duplicate_funcs(filename)
 
+  @staticmethod
+  def calculate_reachable_functions(infile, initial_list, can_reach=True):
+    import asm_module
+    temp = configuration.get_temp_files().get('.js').name
+    Building.js_optimizer(infile, ['dumpCallGraph'], output_filename=temp, just_concat=True)
+    asm = asm_module.AsmModule(temp)
+    lines = asm.funcs_js.split('\n')
+    can_call = {}
+    for i in range(len(lines)):
+      line = lines[i]
+      if line.startswith('// REACHABLE '):
+        curr = json.loads(line[len('// REACHABLE '):])
+        func = curr[0]
+        targets = curr[2]
+        can_call[func] = set(targets)
+    # function tables too - treat a function all as a function that can call anything in it, which is effectively what it is
+    for name, funcs in asm.tables.iteritems():
+      can_call[name] = set(map(lambda x: x.strip(), funcs[1:-1].split(',')))
+    #print can_call
+    # Note: We ignore calls in from outside the asm module, so you could do emterpreted => outside => emterpreted, and we would
+    #       miss the first one there. But this is acceptable to do, because we can't save such a stack anyhow, due to the outside!
+    #print 'can call', can_call, '\n!!!\n', asm.tables, '!'
+    reachable_from = {}
+    for func, targets in can_call.iteritems():
+      for target in targets:
+        if target not in reachable_from:
+          reachable_from[target] = set()
+        reachable_from[target].add(func)
+    #print 'reachable from', reachable_from
+    to_check = initial_list[:]
+    advised = set()
+    if can_reach:
+      # find all functions that can reach the initial list
+      while len(to_check) > 0:
+        curr = to_check.pop()
+        if curr in reachable_from:
+          for reacher in reachable_from[curr]:
+            if reacher not in advised:
+              if not JS.is_dyn_call(reacher) and not JS.is_function_table(reacher): advised.add(str(reacher))
+              to_check.append(reacher)
+    else:
+      # find all functions that are reachable from the initial list, including it
+      # all tables are assumed reachable, as they can be called from dyncall from outside
+      for name, funcs in asm.tables.iteritems():
+        to_check.append(name)
+      while len(to_check) > 0:
+        curr = to_check.pop()
+        if not JS.is_function_table(curr):
+          advised.add(curr)
+        if curr in can_call:
+          for target in can_call[curr]:
+            if target not in advised:
+              advised.add(str(target))
+              to_check.append(target)
+    return { 'reachable': list(advised), 'total_funcs': len(can_call) }
+
   @staticmethod
   def closure_compiler(filename, pretty=True):
     if not check_closure_compiler():
@@ -1941,6 +2009,14 @@ class JS:
     def escape(x): return '\\x{:02x}'.format(ord(x.group()))
     return re.sub('[\x1a\x80-\xff]', escape, s)
 
+  @staticmethod
+  def is_dyn_call(func):
+    return func.startswith('dynCall_')
+
+  @staticmethod
+  def is_function_table(name):
+    return name.startswith('FUNCTION_TABLE_')
+
 def execute(cmd, *args, **kw):
   try:
     return Popen(cmd, *args, **kw).communicate() # let compiler frontend print directly, so colors are saved (PIPE kills that)
